/**
 * Copyright (c) 2016-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.so;

import static org.junit.Assert.assertEquals;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.junit.Test;
import org.ow2.petals.cli.api.shell.Shell;
import org.ow2.petals.cli.extension.command.monitoring.so.api.SubscriptionObject;
import org.ow2.petals.cli.extension.command.monitoring.so.api.SubscriptionObjectRegistration;
import org.ow2.petals.cli.extension.command.monitoring.so.api.SubscriptionObjectRegistror;
import org.ow2.petals.cli.extension.command.monitoring.so.api.exception.DuplicatedEmittableDefectException;
import org.ow2.petals.cli.extension.command.monitoring.so.api.exception.DuplicatedSubscriptionObjectException;
import org.ow2.petals.cli.extension.command.monitoring.so.api.exception.InvalidEmittableDefectNameException;
import org.ow2.petals.cli.extension.command.monitoring.so.api.exception.InvalidSubscriptionObjectNameException;
import org.ow2.petals.cli.extension.command.monitoring.so.components.bc.soap.BcSoap;
import org.ow2.petals.cli.extension.command.monitoring.so.components.framework.CdkBasedComponent;
import org.ow2.petals.cli.extension.command.monitoring.so.components.se.flowable.SeFlowable;

/**
 * A unit test to check that Monitoring extension of Petals ESB CLI embeds all its subscription objects, and only the
 * expected ones.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class EmbeddedSubscriptionObjectsTest {

    private class MySubscriptionObjectRegistror implements SubscriptionObjectRegistror {

        final private Map<String, SubscriptionObject> subscriptionObjects = new HashMap<>();

        @Override
        public void registersSubscriptionObjects(final SubscriptionObject monitoringObject)
                throws DuplicatedSubscriptionObjectException, DuplicatedEmittableDefectException,
                InvalidSubscriptionObjectNameException, InvalidEmittableDefectNameException, IllegalArgumentException {
            this.subscriptionObjects.put(monitoringObject.getName(), monitoringObject);
        }

        @Override
        public Shell getShell() {
            return null;
        }

        @Override
        public Map<String, SubscriptionObject> getSubscriptionObjects() {
            return this.subscriptionObjects;
        }
    }

    private final static List<Class<? extends SubscriptionObject>> EXPECTED_SUBSCRIPTION_OBJECTS;

    static {
        EXPECTED_SUBSCRIPTION_OBJECTS = new ArrayList<>();
        EXPECTED_SUBSCRIPTION_OBJECTS.add(CdkBasedComponent.class);
        EXPECTED_SUBSCRIPTION_OBJECTS.add(BcSoap.class);
        EXPECTED_SUBSCRIPTION_OBJECTS.add(SeFlowable.class);
    }

    @Test
    public void expectedSubscriptionObjects() {

        final SubscriptionObjectRegistror registror = new MySubscriptionObjectRegistror();
        SubscriptionObjectRegistration.registers(registror);

        final List<Class<? extends SubscriptionObject>> expectedSubscriptionObjects = new ArrayList<>(
                EXPECTED_SUBSCRIPTION_OBJECTS);

        final Map<String, SubscriptionObject> subscriptionObjects = registror.getSubscriptionObjects();
        final Iterator<SubscriptionObject> itSubscriptionObjects = subscriptionObjects.values().iterator();
        while (itSubscriptionObjects.hasNext()) {
            final SubscriptionObject monitoringObject = itSubscriptionObjects.next();

            final Iterator<Class<? extends SubscriptionObject>> itExpectedSubscriptionObjects = expectedSubscriptionObjects
                    .iterator();
            while (itExpectedSubscriptionObjects.hasNext()) {
                final Class<? extends SubscriptionObject> expectedSubscriptionObject = itExpectedSubscriptionObjects
                        .next();
                if (expectedSubscriptionObject.isInstance(monitoringObject)) {
                    itExpectedSubscriptionObjects.remove();
                    itSubscriptionObjects.remove();
                    break;
                }
            }
        }

        assertEquals("Few subscription objects are not expected", 0, subscriptionObjects.size());
        assertEquals("Few subscription objects are missing", 0, expectedSubscriptionObjects.size());
    }

}
